/*
 * Copyright (c) 2007 Christian W. Guenther (christian@deckfour.org)
 * 
 * LICENSE:
 * 
 * This code is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 * 
 * EXEMPTION:
 * 
 * License to link and use is also granted to open source programs which are not
 * licensed under the terms of the GPL, given that they satisfy one or more of
 * the following conditions: 1) Explicit license is granted to the ProM and
 * ProMimport programs for usage, linking, and derivative work. 2) Carte blance
 * license is granted to all programs developed at Eindhoven Technical
 * University, The Netherlands, or under the umbrella of STW Technology
 * Foundation, The Netherlands. For further exemptions not covered by the above
 * conditions, please contact the author of this code.
 */
package org.processmining.plugins.ltlchecker;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.ScrollPaneConstants;
import javax.swing.SwingConstants;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XTrace;

import com.fluxicon.slickerbox.components.RoundedPanel;
import com.fluxicon.slickerbox.ui.SlickerScrollBarUI;

/**
 * @author Fabrizio M. Maggi
 * 
 */
public class CheckResultsInstanceBrowser extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = 7772305278301666972L;
	public static final Color COLOR_ENCLOSURE_BG = new Color(40, 40, 40);
	public static final Color COLOR_NON_FOCUS = new Color(70, 70, 70);
	public static final Color COLOR_LIST_BG = new Color(60, 60, 60);
	public static final Color COLOR_LIST_BG_LOWER = new Color(45, 45, 45);
	public static final Color COLOR_LIST_FG = new Color(180, 180, 180);
	public static final Color COLOR_LIST_SELECTION_BG = new Color(80, 0, 0);
	public static final Color COLOR_LIST_SELECTION_BG_LOWER = new Color(30, 10, 10);
	public static final Color COLOR_LIST_SELECTION_FG = new Color(240, 240, 240);

	protected JScrollPane rulesScrollPane;
	protected JList instancesList;
	protected JLabel eventsNameLabel;
	protected JLabel eventsSizeLabel;
	protected JList eventsList;
	protected JLabel rulesLabel;
	protected JList rulesList;
	protected CheckResultObject crObj;
	protected Vector filteredInstances;
	protected Vector indexes;
	protected boolean good = false;

	public CheckResultsInstanceBrowser(CheckResultObject crObj, boolean good) {
		this.crObj = crObj;
		filteredInstances = new Vector();
		indexes = new Vector();
		if (good) {
			this.good = true;
		}
		InstanceModel[] imArr = crObj.getInstances();
		for (int i = 0; i < imArr.length; i++) {
			if (imArr[i].isCorrect() && good) {
				filteredInstances.add(imArr[i]);
				indexes.add(new Integer(i));
			} else if (!imArr[i].isCorrect() && !good) {
				filteredInstances.add(imArr[i]);
				indexes.add(new Integer(i));
			}
		}
		setOpaque(false);
		setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
		// create instances list
		instancesList = new JList(filteredInstances);
		instancesList.setBackground(COLOR_LIST_BG);
		instancesList.setForeground(COLOR_LIST_FG);
		instancesList.setSelectionBackground(COLOR_LIST_SELECTION_BG);
		instancesList.setSelectionForeground(COLOR_LIST_SELECTION_FG);
		instancesList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		instancesList.addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {
				instancesSelectionChanged();
			}
		});
		String[] instanceNames = new String[filteredInstances.size()];
		for (int i = 0; i < instanceNames.length; i++) {
			instanceNames[i] = XConceptExtension.instance().extractName(
					((InstanceModel) filteredInstances.get(i)).getInstance());
		}
		instancesList.setListData(instanceNames);
		// reset events list
		instancesList.clearSelection();
		JScrollPane instancesScrollPane = new JScrollPane(instancesList);
		instancesScrollPane.setOpaque(false);
		instancesScrollPane.setBorder(BorderFactory.createEmptyBorder());
		instancesScrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
		instancesScrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		JScrollBar vBar = instancesScrollPane.getVerticalScrollBar();
		vBar.setUI(new SlickerScrollBarUI(vBar, new Color(0, 0, 0, 0), new Color(160, 160, 160), COLOR_NON_FOCUS, 4, 12));
		vBar.setOpaque(false);
		vBar = instancesScrollPane.getHorizontalScrollBar();
		vBar.setUI(new SlickerScrollBarUI(vBar, new Color(0, 0, 0, 0), new Color(200, 200, 200), COLOR_NON_FOCUS, 4, 12));
		vBar.setOpaque(false);
		// assemble instances list
		JLabel instancesListLabel = new JLabel("Instances");
		instancesListLabel.setOpaque(false);
		instancesListLabel.setForeground(COLOR_LIST_SELECTION_FG);
		instancesListLabel.setFont(instancesListLabel.getFont().deriveFont(13f));
		instancesListLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		instancesListLabel.setHorizontalAlignment(SwingConstants.CENTER);
		instancesListLabel.setHorizontalTextPosition(SwingConstants.CENTER);
		RoundedPanel instancesPanel = new RoundedPanel(10, 5, 0);
		instancesPanel.setBackground(COLOR_ENCLOSURE_BG);
		instancesPanel.setLayout(new BoxLayout(instancesPanel, BoxLayout.Y_AXIS));
		instancesPanel.setMaximumSize(new Dimension(180, 1000));
		instancesPanel.setPreferredSize(new Dimension(180, 1000));
		instancesPanel.add(instancesListLabel);
		instancesPanel.add(Box.createVerticalStrut(8));
		instancesPanel.add(instancesScrollPane);
		// create event list header
		eventsNameLabel = new JLabel("(no instance selected)");
		eventsNameLabel.setOpaque(false);
		eventsNameLabel.setForeground(COLOR_NON_FOCUS);
		eventsNameLabel.setFont(eventsNameLabel.getFont().deriveFont(13f));
		eventsNameLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		eventsNameLabel.setHorizontalAlignment(SwingConstants.CENTER);
		eventsNameLabel.setHorizontalTextPosition(SwingConstants.CENTER);
		eventsSizeLabel = new JLabel("select single instance to browse");
		eventsSizeLabel.setOpaque(false);
		eventsSizeLabel.setForeground(COLOR_NON_FOCUS);
		eventsSizeLabel.setFont(eventsSizeLabel.getFont().deriveFont(11f));
		eventsSizeLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		eventsSizeLabel.setHorizontalAlignment(SwingConstants.CENTER);
		eventsSizeLabel.setHorizontalTextPosition(SwingConstants.CENTER);
		// create events list and result list
		eventsList = new JList();
		rulesList = new JList();
		rulesList.setBackground(COLOR_LIST_BG);
		rulesList.setCellRenderer(new InstancesCellRenderer());
		rulesList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		eventsList.setBackground(COLOR_LIST_BG);
		eventsList.setCellRenderer(new EventCellRendererLC());
		eventsList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		JScrollPane eventsScrollPane = new JScrollPane(eventsList);
		eventsScrollPane.setOpaque(false);
		eventsScrollPane.setBorder(BorderFactory.createEmptyBorder());
		eventsScrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
		eventsScrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		vBar = eventsScrollPane.getVerticalScrollBar();
		vBar.setUI(new SlickerScrollBarUI(vBar, new Color(0, 0, 0, 0), new Color(160, 160, 160), COLOR_NON_FOCUS, 4, 12));
		vBar.setOpaque(false);
		vBar = eventsScrollPane.getHorizontalScrollBar();
		vBar.setUI(new SlickerScrollBarUI(vBar, new Color(0, 0, 0, 0), new Color(200, 200, 200), COLOR_NON_FOCUS, 4, 12));
		vBar.setOpaque(false);
		// assemble events list
		RoundedPanel eventsPanel = new RoundedPanel(10, 5, 0);
		eventsPanel.setBackground(COLOR_ENCLOSURE_BG);
		eventsPanel.setLayout(new BoxLayout(eventsPanel, BoxLayout.Y_AXIS));
		eventsPanel.add(eventsNameLabel);
		eventsPanel.add(eventsSizeLabel);
		eventsPanel.add(Box.createVerticalStrut(8));
		eventsPanel.add(eventsScrollPane);
		eventsPanel.setMinimumSize(new Dimension(300, 1000));
		eventsPanel.setSize(300, 1000);
		eventsPanel.setMaximumSize(new Dimension(300, 1000));
		eventsPanel.setPreferredSize(new Dimension(300, 1000));
		rulesScrollPane = new JScrollPane(rulesList);
		rulesScrollPane.setOpaque(false);
		rulesScrollPane.setBorder(BorderFactory.createEmptyBorder());
		rulesScrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
		rulesScrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		vBar = rulesScrollPane.getVerticalScrollBar();
		vBar.setUI(new SlickerScrollBarUI(vBar, new Color(0, 0, 0, 0), new Color(200, 200, 200), COLOR_NON_FOCUS, 4, 12));
		vBar.setOpaque(false);
		vBar = rulesScrollPane.getHorizontalScrollBar();
		vBar.setUI(new SlickerScrollBarUI(vBar, new Color(0, 0, 0, 0), new Color(200, 200, 200), COLOR_NON_FOCUS, 4, 12));
		vBar.setOpaque(false);
		// assemble attributes panel
		rulesLabel = new JLabel("(no instance selected)");
		rulesLabel.setOpaque(false);
		rulesLabel.setForeground(COLOR_NON_FOCUS);
		rulesLabel.setFont(rulesLabel.getFont().deriveFont(13f));
		rulesLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		rulesLabel.setHorizontalAlignment(SwingConstants.CENTER);
		rulesLabel.setHorizontalTextPosition(SwingConstants.CENTER);
		JPanel rulesPanel = new RoundedPanel(10, 5, 0);
		rulesPanel.setBackground(COLOR_ENCLOSURE_BG);
		rulesPanel.setLayout(new BoxLayout(rulesPanel, BoxLayout.Y_AXIS));
		rulesPanel.setMaximumSize(new Dimension(2000, 2000));
		rulesPanel.setPreferredSize(new Dimension(2000, 2000));
		rulesPanel.add(rulesLabel);
		rulesPanel.add(Box.createVerticalStrut(8));
		rulesPanel.add(rulesScrollPane);
		// assemble GUI

		this.add(instancesPanel);
		this.add(rulesPanel);
		this.add(eventsPanel);

	}

	protected void instancesSelectionChanged() {
		int[] selectedIndices = instancesList.getSelectedIndices();
		if ((selectedIndices.length == 0) || (selectedIndices.length > 1)) {
			eventsList.setCellRenderer(new EventCellRendererLC());
			eventsList.repaint();
			eventsList.setListData(new Object[] {});
			eventsList.clearSelection();
			eventsNameLabel.setForeground(COLOR_NON_FOCUS);
			eventsNameLabel.setText("(no instance selected)");
			eventsSizeLabel.setForeground(COLOR_NON_FOCUS);
			eventsSizeLabel.setText("select single instance to browse");
		} else {
			eventsList.setCellRenderer(new EventCellRendererLC());
			eventsList.repaint();
			XTrace instance = ((InstanceModel) filteredInstances.get(selectedIndices[0])).getInstance();
			eventsNameLabel.setForeground(COLOR_LIST_SELECTION_FG);
			eventsNameLabel.setText(XConceptExtension.instance().extractName(instance));
			eventsSizeLabel.setForeground(COLOR_LIST_SELECTION_FG);
			eventsSizeLabel.setText(instance.size() + " events");
			eventsList.setModel(new XTraceListModel(instance));
			eventsList.ensureIndexIsVisible(0);
		}
		showSelectedInstanceData();
	}

	protected void eventsSelectionChanged() {
		XEvent ate = (XEvent) eventsList.getSelectedValue();
		if (ate != null) {
			rulesLabel.setForeground(COLOR_LIST_SELECTION_FG);
			rulesLabel.setText("Attributes for event " + (eventsList.getSelectedIndex() + 1));
		} else {
			showSelectedInstanceData();
		}
	}

	protected void showSelectedInstanceData() {
		int[] selectedIndices = instancesList.getSelectedIndices();
		if (selectedIndices.length == 1) {
			InstanceModel instance = (InstanceModel) filteredInstances.get(selectedIndices[0]);
			rulesLabel.setForeground(COLOR_LIST_SELECTION_FG);
			double coverage = ((int) (instance.getHealthDegree() * 100)) / 100.0;
			if (coverage >= 0) {
				rulesLabel.setText("Health degree for case "
						+ XConceptExtension.instance().extractName(instance.getInstance()) + " = " + coverage);
			} else {
				rulesLabel.setText("Health degree for case "
						+ XConceptExtension.instance().extractName(instance.getInstance()) + " = N.A.");
			}
			Vector temp = new Vector();
			for (int i = 0; i < crObj.getRules().length; i++) {
				CheckResultCell crCell = new CheckResultCell();
				RuleModel rm = crObj.getRules()[i];
				crCell.setInstance(instance.getInstance());
				crCell.setLtl(rm.getLtlRule());
				crCell.setRule(rm.getRuleName());
				crCell.setTruthValue(crObj.getMatrix()[i][((Integer) indexes.get(selectedIndices[0])).intValue()]);
				temp.add(crCell);
			}
			rulesList.setModel(new CheckListModel(temp));
		} else {
			rulesLabel.setForeground(COLOR_NON_FOCUS);
			rulesLabel.setText("(no case or event selected)");
			rulesList.setModel(new CheckListModel(null));
		}
	}
}
