/*
 * Copyright (c) 2007 Christian W. Guenther (christian@deckfour.org)
 * 
 * LICENSE:
 * 
 * This code is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 * 
 * EXEMPTION:
 * 
 * License to link and use is also granted to open source programs which are not
 * licensed under the terms of the GPL, given that they satisfy one or more of
 * the following conditions: 1) Explicit license is granted to the ProM and
 * ProMimport programs for usage, linking, and derivative work. 2) Carte blance
 * license is granted to all programs developed at Eindhoven Technical
 * University, The Netherlands, or under the umbrella of STW Technology
 * Foundation, The Netherlands. For further exemptions not covered by the above
 * conditions, please contact the author of this code.
 */
package org.processmining.plugins.ltlchecker;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.ScrollPaneConstants;
import javax.swing.SwingConstants;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.deckfour.xes.extension.std.XConceptExtension;

import com.fluxicon.slickerbox.components.RoundedPanel;
import com.fluxicon.slickerbox.ui.SlickerScrollBarUI;

/**
 * @author Fabrizio M. Maggi
 * 
 */
public class CheckResultsRuleBrowser extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = 7772305278301666972L;
	public static final Color COLOR_ENCLOSURE_BG = new Color(40, 40, 40);
	public static final Color COLOR_NON_FOCUS = new Color(70, 70, 70);
	public static final Color COLOR_LIST_BG = new Color(60, 60, 60);
	public static final Color COLOR_LIST_BG_LOWER = new Color(45, 45, 45);
	public static final Color COLOR_LIST_FG = new Color(180, 180, 180);
	public static final Color COLOR_LIST_SELECTION_BG = new Color(80, 0, 0);
	public static final Color COLOR_LIST_SELECTION_BG_LOWER = new Color(30, 10, 10);
	public static final Color COLOR_LIST_SELECTION_FG = new Color(240, 240, 240);

	protected JScrollPane caseScrollPane;
	protected JList rulesList;
	protected JLabel instanceNameLabel;
	protected JLabel instanceSizeLabel;
	protected JList eventsList;
	protected JLabel caseLabel;
	protected JList caseList;
	protected CheckResultObject crObj;
	protected Vector indexes;
	protected Vector filteredRules;

	public CheckResultsRuleBrowser(CheckResultObject crObj, boolean good) {
		this.crObj = crObj;
		filteredRules = new Vector();
		RuleModel[] rmArr = crObj.getRules();
		indexes = new Vector();
		for (int i = 0; i < rmArr.length; i++) {
			if (rmArr[i].isSatisfied() && good) {
				filteredRules.add(rmArr[i]);
				indexes.add(new Integer(i));
			} else if (!rmArr[i].isSatisfied() && !good) {
				filteredRules.add(rmArr[i]);
				indexes.add(new Integer(i));
			}
		}
		setOpaque(false);
		setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
		// create instances list
		rulesList = new JList();
		rulesList.setBackground(COLOR_LIST_BG);
		rulesList.setForeground(COLOR_LIST_FG);
		rulesList.setCellRenderer(new RulesCellRenderer());
		rulesList.setModel(new CheckListModel(filteredRules));
		rulesList.setSelectionBackground(COLOR_LIST_SELECTION_BG);
		rulesList.setSelectionForeground(COLOR_LIST_SELECTION_FG);
		rulesList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		rulesList.addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {
				instancesSelectionChanged();
			}
		});
		// reset events list
		rulesList.clearSelection();
		JScrollPane rulesScrollPane = new JScrollPane(rulesList);
		rulesScrollPane.setOpaque(false);
		rulesScrollPane.setBorder(BorderFactory.createEmptyBorder());
		rulesScrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
		rulesScrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		JScrollBar vBar = rulesScrollPane.getVerticalScrollBar();
		vBar.setUI(new SlickerScrollBarUI(vBar, new Color(0, 0, 0, 0), new Color(160, 160, 160), COLOR_NON_FOCUS, 4, 12));
		vBar.setOpaque(false);
		vBar = rulesScrollPane.getHorizontalScrollBar();
		vBar.setUI(new SlickerScrollBarUI(vBar, new Color(0, 0, 0, 0), new Color(200, 200, 200), COLOR_NON_FOCUS, 4, 12));
		vBar.setOpaque(false);
		// assemble instances list
		JLabel rulesListLabel = new JLabel("Rules");
		rulesListLabel.setOpaque(false);
		rulesListLabel.setForeground(COLOR_LIST_SELECTION_FG);
		rulesListLabel.setFont(rulesListLabel.getFont().deriveFont(13f));
		rulesListLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		rulesListLabel.setHorizontalAlignment(SwingConstants.CENTER);
		rulesListLabel.setHorizontalTextPosition(SwingConstants.CENTER);
		RoundedPanel rulesPanel = new RoundedPanel(10, 5, 0);
		rulesPanel.setBackground(COLOR_ENCLOSURE_BG);
		rulesPanel.setLayout(new BoxLayout(rulesPanel, BoxLayout.Y_AXIS));
		rulesPanel.setMaximumSize(new Dimension(1500, 1000));
		rulesPanel.setPreferredSize(new Dimension(1500, 1000));
		rulesPanel.add(rulesListLabel);
		rulesPanel.add(Box.createVerticalStrut(8));
		rulesPanel.add(rulesScrollPane);
		// create event list header
		instanceNameLabel = new JLabel("(no instance selected)");
		instanceNameLabel.setOpaque(false);
		instanceNameLabel.setForeground(COLOR_NON_FOCUS);
		instanceNameLabel.setFont(instanceNameLabel.getFont().deriveFont(13f));
		instanceNameLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		instanceNameLabel.setHorizontalAlignment(SwingConstants.CENTER);
		instanceNameLabel.setHorizontalTextPosition(SwingConstants.CENTER);
		instanceSizeLabel = new JLabel("select single case to browse");
		instanceSizeLabel.setOpaque(false);
		instanceSizeLabel.setForeground(COLOR_NON_FOCUS);
		instanceSizeLabel.setFont(instanceSizeLabel.getFont().deriveFont(11f));
		instanceSizeLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		instanceSizeLabel.setHorizontalAlignment(SwingConstants.CENTER);
		instanceSizeLabel.setHorizontalTextPosition(SwingConstants.CENTER);
		// create events list and result list
		eventsList = new JList();
		caseList = new JList();
		caseList.setBackground(COLOR_LIST_BG);
		caseList.setCellRenderer(new RulesInstancesCellRenderer());
		caseList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		caseList.addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {
				eventsSelectionChanged();
			}
		});
		eventsList.setBackground(COLOR_LIST_BG);
		eventsList.setCellRenderer(new EventCellRendererLC());
		eventsList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		JScrollPane eventsScrollPane = new JScrollPane(eventsList);
		eventsScrollPane.setOpaque(false);
		eventsScrollPane.setBorder(BorderFactory.createEmptyBorder());
		eventsScrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
		eventsScrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		vBar = eventsScrollPane.getVerticalScrollBar();
		vBar.setUI(new SlickerScrollBarUI(vBar, new Color(0, 0, 0, 0), new Color(160, 160, 160), COLOR_NON_FOCUS, 4, 12));
		vBar.setOpaque(false);
		vBar = eventsScrollPane.getHorizontalScrollBar();
		vBar.setUI(new SlickerScrollBarUI(vBar, new Color(0, 0, 0, 0), new Color(200, 200, 200), COLOR_NON_FOCUS, 4, 12));
		vBar.setOpaque(false);
		// assemble events list
		RoundedPanel eventsPanel = new RoundedPanel(10, 5, 0);
		eventsPanel.setBackground(COLOR_ENCLOSURE_BG);
		eventsPanel.setLayout(new BoxLayout(eventsPanel, BoxLayout.Y_AXIS));
		eventsPanel.add(instanceNameLabel);
		eventsPanel.add(instanceSizeLabel);
		eventsPanel.add(Box.createVerticalStrut(8));
		eventsPanel.add(eventsScrollPane);
		eventsPanel.setMinimumSize(new Dimension(300, 1000));
		eventsPanel.setSize(300, 1000);
		eventsPanel.setMaximumSize(new Dimension(300, 1000));
		eventsPanel.setPreferredSize(new Dimension(300, 1000));
		caseScrollPane = new JScrollPane(caseList);
		caseScrollPane.setOpaque(false);
		caseScrollPane.setBorder(BorderFactory.createEmptyBorder());
		caseScrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
		caseScrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		vBar = caseScrollPane.getVerticalScrollBar();
		vBar.setUI(new SlickerScrollBarUI(vBar, new Color(0, 0, 0, 0), new Color(200, 200, 200), COLOR_NON_FOCUS, 4, 12));
		vBar.setOpaque(false);
		vBar = caseScrollPane.getHorizontalScrollBar();
		vBar.setUI(new SlickerScrollBarUI(vBar, new Color(0, 0, 0, 0), new Color(200, 200, 200), COLOR_NON_FOCUS, 4, 12));
		vBar.setOpaque(false);

		// assemble attributes panel
		caseLabel = new JLabel("(no rule selected)");
		caseLabel.setOpaque(false);
		caseLabel.setForeground(COLOR_NON_FOCUS);
		caseLabel.setFont(caseLabel.getFont().deriveFont(13f));
		caseLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		caseLabel.setHorizontalAlignment(SwingConstants.CENTER);
		caseLabel.setHorizontalTextPosition(SwingConstants.CENTER);
		JPanel casePanel = new RoundedPanel(10, 5, 0);
		casePanel.setBackground(COLOR_ENCLOSURE_BG);
		casePanel.setLayout(new BoxLayout(casePanel, BoxLayout.Y_AXIS));
		casePanel.setMinimumSize(new Dimension(200, 1000));
		casePanel.setSize(200, 1000);
		casePanel.setMaximumSize(new Dimension(200, 1000));
		casePanel.setPreferredSize(new Dimension(200, 1000));
		casePanel.add(caseLabel);
		casePanel.add(Box.createVerticalStrut(8));
		casePanel.add(caseScrollPane);

		// assemble GUI
		this.add(rulesPanel);
		this.add(casePanel);
		this.add(eventsPanel);
	}

	protected void instancesSelectionChanged() {
		int[] selectedIndices = rulesList.getSelectedIndices();
		if ((selectedIndices.length == 0) || (selectedIndices.length > 1)) {
			eventsList.setListData(new Object[] {});
			eventsList.clearSelection();
			instanceNameLabel.setForeground(COLOR_NON_FOCUS);
			instanceNameLabel.setText("(no instance selected)");
			instanceSizeLabel.setForeground(COLOR_NON_FOCUS);
			instanceSizeLabel.setText("select single instance to browse");
		} else {
			//RuleModel instance = (RuleModel)rules.get(selectedIndices[0]);
			instanceNameLabel.setForeground(COLOR_LIST_SELECTION_FG);
			//	instanceNameLabel.setText(XConceptExtension.instance().extractName(((CheckResultObject)filteredRules.get(selectedIndices[0])).getInstance()));
			instanceSizeLabel.setForeground(COLOR_LIST_SELECTION_FG);
			//	instanceSizeLabel.setText(((CheckResultObject)filteredRules.get(selectedIndices[0])).getInstance().size() + " events");
			//eventsList.setModel(new XTraceListModel(instance));
			//eventsList.ensureIndexIsVisible(0);
		}
		showSelectedInstanceData();
	}

	protected void eventsSelectionChanged() {
		CheckResultCell ate = (CheckResultCell) caseList.getSelectedValue();

		if (ate != null) {

			//	eventLabel.setForeground(COLOR_LIST_SELECTION_FG);
			//	eventLabel.setText("Attributes for event " + (eventsList.getSelectedIndex() + 1));
			eventsList.setModel(new XTraceListModel(ate.getInstance()));
			eventsList.ensureIndexIsVisible(0);
			instanceNameLabel.setText(XConceptExtension.instance().extractName(ate.getInstance()));
			instanceSizeLabel.setText(ate.getInstance().size() + " events");
			//	attributesList.setModel(new AttributesListModel(ate));
			//eventsList.getCellRenderer().getListCellRendererComponent(null, null , eventsList.getSelectedIndex() + 1, false, false);
			//eventLabel.setText("Rules satisfaction for case " + XConceptExtension.instance().extractName(ate));
		} else {
			showSelectedInstanceData();
		}
	}

	protected void showSelectedInstanceData() {
		int[] selectedIndices = rulesList.getSelectedIndices();

		if (selectedIndices.length == 1) {
			RuleModel rule = (RuleModel) filteredRules.get(selectedIndices[0]);
			//eventsList.setListData(listData)
			caseLabel.setForeground(COLOR_LIST_SELECTION_FG);
			//	eventLabel.setText("Coverage = "+ Approssimator.approssimate(rule.getCoverage(),2));
			double coverage = ((int) (rule.getCoverage() * 100)) / 100.0;
			if (coverage >= 0) {
				caseLabel.setText("Coverage = " + coverage);
			} else {
				caseLabel.setText("Coverage = N.A.");
			}
			Vector temp = new Vector();

			for (int j = 0; j < crObj.getInstances().length; j++) {
				CheckResultCell crCell = new CheckResultCell();
				InstanceModel im = crObj.getInstances()[j];
				crCell.setInstance(im.getInstance());
				crCell.setLtl(rule.getLtlRule());
				crCell.setRule(rule.getRuleName());
				crCell.setTruthValue(crObj.getMatrix()[((Integer) indexes.get(selectedIndices[0])).intValue()][j]);
				temp.add(crCell);
				//		XTrace currTracei = ((CheckResultObject)crObjList.get(i)).getInstance();
				//			 if(XConceptExtension.instance().extractName(currTracei).equals(XConceptExtension.instance().extractName(instance))){

				//		 }
			}

			//	}
			/*
			 * 
			 * for(int i = 0 ; i<crObjList.size(); i++){ String currRulei =
			 * ((CheckResultObject)crObjList.get(i)).getRule();
			 * if(currRulei.equals(rule.getRule())){
			 * temp.add((CheckResultObject)crObjList.get(i)); } }
			 */

			/*
			 * rulesResult.clear(); res.clear(); int index = -1; for(int j =0;
			 * j<instances.size(); j++){
			 * if(instances.get(j).equals(filteredInstances
			 * .get(selectedIndices[0]))){ index = j; } }
			 * 
			 * //Vector res = new Vector(); for(int i=0;i < matrix.length; i++){
			 * //boolean gr = true; if(matrix[i][index]){
			 * rulesResult.add(rules.get(i)); res.add("true"); }else{
			 * 
			 * rulesResult.add(rules.get(i)); res.add("false"); }
			 * 
			 * }
			 * 
			 * centerPanel.add(new JList(rulesResult)); //
			 * centerPanel.setBackground(COLOR_LIST_BG); //
			 * centerPanel.setForeground(COLOR_LIST_FG); centerPanel.validate();
			 * centerPanel.repaint();
			 */
			//resultsList.setListData(res);
			/*
			 * Vector temp = new Vector(); for(int i = 0 ; i<crObjList.size();
			 * i++){ XTrace currTracei =
			 * ((CheckResultObject)crObjList.get(i)).getInstance();
			 * if(XConceptExtension
			 * .instance().extractName(currTracei).equals(XConceptExtension
			 * .instance().extractName(instance))){
			 * temp.add((CheckResultObject)crObjList.get(i)); } }
			 */

			caseList.setModel(new CheckListModel(temp));
			//	attributesScrollPane.validate();
			//	attributesScrollPane.repaint();
			//	attributesPanel.validate();
			//	attributesPanel.repaint();
		} else {
			caseLabel.setForeground(COLOR_NON_FOCUS);
			caseLabel.setText("(no case or event selected)");
			//attributesList.setModel(new AttributesListModel(instance));
			caseList.setModel(new CheckListModel(null));
		}
	}

}
