/***********************************************************
 * This software is part of the ProM package * http://www.processmining.org/ * *
 * Copyright (c) 2003-2006 TU/e Eindhoven * and is licensed under the * Common
 * Public License, Version 1.0 * by Eindhoven University of Technology *
 * Department of Information Systems * http://is.tm.tue.nl * *
 **********************************************************/

package org.processmining.plugins.ltlchecker;

import info.clearthought.layout.TableLayout;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JEditorPane;
import javax.swing.JInternalFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.border.TitledBorder;
import javax.swing.event.CellEditorListener;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.deckfour.xes.model.XLog;
import org.processmining.plugins.ltlchecker.formulatree.FormulaNode;
import org.processmining.plugins.ltlchecker.formulatree.RootNode;
import org.processmining.plugins.ltlchecker.formulatree.ValueNode;
import org.processmining.plugins.ltlchecker.parser.ASTformulaDefinition;
import org.processmining.plugins.ltlchecker.parser.FormulaParameter;
import org.processmining.plugins.ltlchecker.parser.LTLParser;
import org.processmining.plugins.ltlchecker.parser.SimpleNode;
import org.processmining.plugins.ltlchecker.util.ParamData;
import org.processmining.plugins.ltlchecker.util.ParamTable;
import org.processmining.plugins.ltlchecker.util.SetsSet;
import org.processmining.plugins.ltlchecker.util.Substitutes;
import org.processmining.plugins.ltlchecker.util.TreeBuilder;

import com.fluxicon.slickerbox.factory.SlickerDecorator;
import com.fluxicon.slickerbox.factory.SlickerFactory;

/**
 * TemplateGui specifies a gui for selecting an template formula to check, see
 * te description of such formula and to valuate the parameters of the formula.
 * Furthermore some checkoptions can be set. After valuating all parameters, the
 * check can be performed.
 * 
 * @version 0.1
 * @author Fabrizio M. Maggi
 */
@SuppressWarnings({ "serial", "unused" })
public class CheckVisualizerGUI extends JInternalFrame implements ActionListener, MouseListener, KeyListener,
		CellEditorListener {

	// FIELDS
	protected Vector selectedRules = new Vector();
	protected String selected;
	protected String[] rules;

	/** List with formula templates to select from. */
	protected JComboBox formulaList;

	/** A field for the description. */
	protected JEditorPane descriptionPane;

	/** A table for valuate the parameters. */
	protected Hashtable paramTable;
	protected ParamTable selectParamTable;
	/** Check option: till first success. */
	protected boolean firstSuccess;

	/** Check option: till first failure. */
	protected boolean firstFailure;

	/** Check option: skip checked formulas. */
	protected boolean skipReady;
	protected JButton selectAllButton;
	protected JButton deselectAllButton;

	/** Parser with parsedata. */
	protected LTLParser parser;
	protected JList rulesList;
	/** Sets with the sets. */
	protected SetsSet sets;

	protected JPanel mainPanel;

	protected boolean modified; // keep track of whether the formulas in the LTL parser have been changed (deleted, copied, etc)

	protected int selectedIndex;

	protected XLog log;

	// CONSTRUCTORS

	public CheckVisualizerGUI(XLog logreader, LTLParser parser, LTLChecker checker/*
																				 * ,
																				 * AnalysisInputItem
																				 * [
																				 * ]
																				 * inputs
																				 */) {
		init(logreader, parser, checker/* , inputs */);
	}

	protected void init(XLog logreader, LTLParser parser, LTLChecker checker/*
																			 * ,
																			 * AnalysisInputItem
																			 * [
																			 * ]
																			 * inputs
																			 */) {

		this.parser = parser;
		setModified(false);
		log = logreader;
		// Build the setsset
		sets = new SetsSet(parser, log);
		sets.fill(log);
		// Checkoption is `whole process':
		firstSuccess = false;
		firstFailure = false;
		setVisible(true);
	}

	// METHODS

	private String makeHTMLPage(String text) {
		return "<html>\n" + "\t" + "<head></head>\n" + "\t" + "<body>\n" + "\t" + "\t" + text + "\t" + "</body>\n"
				+ "</html>";
	}

	private Object[] initFormulaList() {

		if (formulaList != null) {
			formulaList.removeAllItems();
		}
		Iterator i = parser.getVisibleFormulaNames().iterator();
		int formulaSize = 0;
		while (i.hasNext()) {
			formulaSize++;
			i.next();
		}
		Object[] result = new Object[formulaSize];
		i = parser.getVisibleFormulaNames().iterator();
		int count = 0;
		while (i.hasNext()) {
			result[count] = (String) i.next();
			count++;
		}
		if ((formulaList != null) && (formulaList.getModel().getSize() <= 0)) {
			result = new Object[1];
			result[0] = "no formula";
		}
		if (count == 0) {
			result = new Object[1];
			result[0] = "no formula";
		}
		return result;
	}

	public JPanel createGui() {
		SlickerFactory sf = SlickerFactory.instance();
		Object[] rulesListObj = initFormulaList();
		rules = new String[rulesListObj.length];
		rulesList = new JList();
		ListRenderer renderer = new ListRenderer();
		rulesList.setCellRenderer(renderer);

		Vector cellList = new Vector();
		for (int i = 0; i < rulesListObj.length; i++) {
			Cell cellL = new Cell();
			if (i == 0) {
				cellL.setCurrentSelected(true);
			}
			cellL.setLabel((String) rulesListObj[i]);
			cellL.setSelected(false);
			cellList.add(cellL);
		}
		rulesList.setModel(new CellModel(cellList));
		rulesList.setSelectionBackground(new Color(0, 100, 0));
		rulesList.setSelectedIndex(0); //It's easier to see the focus change
		//if an item is selected.
		rulesList.setBackground(new Color(160, 160, 160));
		rulesList.setSelectionForeground(Color.white);
		rulesList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		rulesList.addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {

				String value = null;
				int y = selectParamTable.getTable().getEditingColumn();
				int x = selectParamTable.getTable().getEditingRow();
				if ((x != -1) && (y != -1)) {
					selectParamTable.getTable().getCellEditor(x, y).stopCellEditing();
					value = (String) selectParamTable.getTable().getValueAt(x, y);
					String formulaName = selected;
					Map<String, String> defaults = new HashMap<String, String>();
					for (Map.Entry<String, ValueNode> item : selectParamTable.getSubstitutes(parser).getAll()
							.entrySet()) {
						defaults.put(item.getKey(), item.getValue().asParseableDefaultValue());

					}
					parser.setDefaultValues(formulaName, defaults);
					ParamData pd2 = createDataModel(parser.getParameters(formulaName));
					pd2.getSubstitutes(parser);
					selectParamTable.setModel(pd2);
					selectParamTable.getTable().repaint();
					SimpleNode formula = parser.getFormula(formulaName);

					// Update description this
					ASTformulaDefinition form = (ASTformulaDefinition) formula;
					String description = form.getDescription();
					description = description + " <h2> UNDERLYING LTL FORMULA: </h2> <b> "
							+ getLTLFormula(formulaName).replace("<", "&lt;").replace(">", "&gt;") + "</b>";
					descriptionPane.setText(makeHTMLPage(description));
				}

				if (rulesList.getSelectedIndex() >= 0) {
					String selectedFormula = rules[rulesList.getSelectedIndex()];
					if (selected.equals(rules[rulesList.getSelectedIndex()]) && (value != null)) {
						selectParamTable.getTable().setValueAt(value, x, y);
					}
					selected = rules[rulesList.getSelectedIndex()];
					selectedIndex = rulesList.getSelectedIndex();
					if ((selectedFormula != null) && !selectedFormula.equals("no formula")) {

						SimpleNode formula = parser.getFormula(selectedFormula);

						// Update description this
						ASTformulaDefinition form = (ASTformulaDefinition) formula;
						String description = form.getDescription();
						description = description + "  <h2> UNDERLYING LTL FORMULA: </h2> <b> "
								+ getLTLFormula(selectedFormula).replace("<", "&lt;").replace(">", "&gt;") + "</b>";
						descriptionPane.setText(makeHTMLPage(description));
						// Update param table
						ParamData pd = createDataModel(parser.getParameters(selectedFormula));
						selectParamTable.setModel(pd);
						//				paramTable.put(selectedFormula, selectParamTable);
						//selectParamTable = (ParamTable)paramTable.get(selectedFormula);
						updateRulesList(rulesList.getSelectedIndex());

					}
				}

			}
		});

		rulesList.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {

				//  if (e.getClickCount() == 2) {
				int cellIndex = rulesList.locationToIndex(e.getPoint());
				selected = rules[cellIndex];
				selectedIndex = cellIndex;
				if (!selectedRules.contains(selected)) {
					selectedRules.add(selected);
				} else {
					selectedRules.remove(selected);
				}
				updateRulesList(cellIndex);
				// We now have the index of the double-clicked cell..
				//}
			}
		});

		for (int i = 0; i < rules.length; i++) {
			rules[i] = (String) rulesListObj[i];
		}
		selectedIndex = 0;
		selected = rules[0];

		JScrollPane scrollpane = new JScrollPane(rulesList);
		scrollpane.setOpaque(false);
		scrollpane.getViewport().setOpaque(false);
		scrollpane.setBorder(BorderFactory.createEmptyBorder());
		scrollpane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		scrollpane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
		SlickerDecorator.instance().decorate(scrollpane.getVerticalScrollBar(), new Color(0, 0, 0, 0),
				new Color(140, 140, 140), new Color(80, 80, 80));
		scrollpane.getVerticalScrollBar().setOpaque(false);
		scrollpane.getVerticalScrollBar().setFocusable(false);
		SlickerDecorator.instance().decorate(scrollpane.getHorizontalScrollBar(), new Color(0, 0, 0, 0),
				new Color(140, 140, 140), new Color(80, 80, 80));
		scrollpane.getHorizontalScrollBar().setOpaque(false);
		//JPanel bottomLeftPanel = sf.createRoundedPanel();
		formulaList = sf.createComboBox(initFormulaList());
		formulaList.setActionCommand("formula");
		formulaList.addActionListener(this);
		descriptionPane = new JEditorPane();
		descriptionPane.setEditable(false);
		descriptionPane.addMouseListener(this);
		descriptionPane.setContentType("text/html");
		descriptionPane.setText(makeHTMLPage("<H1>No description available.</H1>"));
		JScrollPane descPane = new JScrollPane(descriptionPane);
		descPane.setOpaque(false);
		descPane.getViewport().setOpaque(false);
		descPane.setBorder(BorderFactory.createEmptyBorder());
		descPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		descPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
		SlickerDecorator.instance().decorate(descPane.getVerticalScrollBar(), new Color(0, 0, 0, 0),
				new Color(140, 140, 140), new Color(80, 80, 80));
		descPane.getVerticalScrollBar().setOpaque(false);

		SlickerDecorator.instance().decorate(descPane.getHorizontalScrollBar(), new Color(0, 0, 0, 0),
				new Color(140, 140, 140), new Color(80, 80, 80));
		descPane.getHorizontalScrollBar().setOpaque(false);

		descPane.setPreferredSize(new Dimension(400, 140));
		descPane.setMinimumSize(new Dimension(400, 140));
		descPane.setMaximumSize(new Dimension(400, 140));
		ParamData pd = createDataModel(new ArrayList());
		selectParamTable = createParamTable();
		selectParamTable.setModel(pd);
		selectParamTable.getTable().addMouseListener(this);
		selectParamTable.getTable().addKeyListener(this);
		selectParamTable.getTable().setEnabled(true);
		paramTable = new Hashtable();
		int height = getParamPaneHeight();
		JEditorPane parPanel = new JEditorPane();
		parPanel.setEditable(false);
		parPanel.setBackground(new Color(238, 238, 238));
		parPanel.addMouseListener(this);
		parPanel.setLayout(new TableLayout(new double[][] { { 390 }, { 50 } }));
		parPanel.add(selectParamTable, "0,0");
		JScrollPane paramPane = new JScrollPane(parPanel);
		paramPane.setOpaque(false);
		paramPane.getViewport().setOpaque(false);
		paramPane.setBorder(BorderFactory.createEmptyBorder());
		paramPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		paramPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
		SlickerDecorator.instance().decorate(descPane.getVerticalScrollBar(), new Color(0, 0, 0, 0),
				new Color(140, 140, 140), new Color(80, 80, 80));
		paramPane.getVerticalScrollBar().setOpaque(false);

		SlickerDecorator.instance().decorate(paramPane.getHorizontalScrollBar(), new Color(0, 0, 0, 0),
				new Color(140, 140, 140), new Color(80, 80, 80));
		paramPane.getHorizontalScrollBar().setOpaque(false);
		paramPane.setPreferredSize(new Dimension(400, height));
		paramPane.setMinimumSize(new Dimension(400, height));
		paramPane.setMaximumSize(new Dimension(400, height));
		paramPane.addMouseListener(this);
		selectParamTable.addMouseListener(this);
		JPanel checkOptionsPanel = createCheckOptionsPanel();
		JEditorPane buttonPanel = createCheckButtonPanel();
		JLabel selectFormulaLabel = sf.createLabel("Select formula:");
		selectFormulaLabel.setLabelFor(formulaList);
		selectFormulaLabel.setDisplayedMnemonic(KeyEvent.VK_F);

		mainPanel = sf.createRoundedPanel();
		mainPanel.setLayout(new TableLayout(new double[][] { { 800 }, { 200, 1, TableLayout.FILL } }));
		JPanel line = sf.createRoundedPanel();
		line.setBackground(new Color(0, 100, 0));
		mainPanel.add(line, "0,1");
		JPanel topPanel = sf.createRoundedPanel();
		topPanel.setLayout(new TableLayout(new double[][] { { 200, 1, 575 }, { 200 } }));
		JPanel verticalLine = sf.createRoundedPanel();
		verticalLine.setBackground(new Color(0, 100, 0));
		topPanel.add(verticalLine, "1,0");
		JEditorPane bottomLeftPanel = new JEditorPane();
		bottomLeftPanel.setEditable(false);
		bottomLeftPanel.addMouseListener(this);
		bottomLeftPanel.setBackground(new Color(160, 160, 160));
		bottomLeftPanel.setLayout(new TableLayout(new double[][] { { 400 }, { 30, TableLayout.FILL } }));
		bottomLeftPanel.add(sf.createLabel("<html>Description :</html>"), "0,0");
		bottomLeftPanel.add(descPane, "0,1");
		//JPanel bottomRightPanel=sf.createRoundedPanel();
		JEditorPane bottomRightPanel = new JEditorPane();
		bottomRightPanel.setEditable(false);
		bottomRightPanel.addMouseListener(this);
		bottomRightPanel.setBackground(new Color(160, 160, 160));
		bottomRightPanel.setLayout(new TableLayout(new double[][] { { 400 }, { 30, TableLayout.FILL } }));
		bottomRightPanel.add(sf.createLabel("<html>Set the parameters :</html>"), "0,0");
		bottomRightPanel.add(paramPane, "0,1");
		JPanel bottomPanel = sf.createRoundedPanel();
		bottomPanel.setLayout(new TableLayout(new double[][] { { 400, 400 }, { TableLayout.FILL } }));
		bottomPanel.add(bottomLeftPanel, "0,0");
		bottomPanel.add(bottomRightPanel, "1,0");
		bottomPanel.addMouseListener(this);
		bottomPanel.addMouseListener(this);
		//JPanel topLeftPanel=sf.createRoundedPanel();
		JEditorPane topLeftPanel = new JEditorPane();
		topLeftPanel.setEditable(false);
		topLeftPanel.addMouseListener(this);
		topLeftPanel.setBackground(new Color(160, 160, 160));
		topLeftPanel.setLayout(new TableLayout(new double[][] { { 400 }, { 50, 150 } }));
		topLeftPanel.add(buttonPanel, "0,0");
		topLeftPanel.add(checkOptionsPanel, "0,1");
		topPanel.add(topLeftPanel, "0,0");
		topPanel.add(scrollpane, "2,0");
		mainPanel.add(topPanel, "0,0");
		mainPanel.add(bottomPanel, "0,2");
		String selectedFormula = (String) formulaList.getSelectedItem();
		if ((selectedFormula != null) && !selectedFormula.equals("no formula")) {
			SimpleNode formula = parser.getFormula(selectedFormula);
			// Update description this
			ASTformulaDefinition form = (ASTformulaDefinition) formula;
			String description = form.getDescription();
			description = description + " <h2> UNDERLYING LTL FORMULA: </h2> <b> "
					+ getLTLFormula(selectedFormula).replace("<", "&lt;").replace(">", "&gt;") + "</b>";
			descriptionPane.setText(makeHTMLPage(description));
			// Update param table
			ParamData pd2 = createDataModel(parser.getParameters(selectedFormula));
			selectParamTable.setModel(pd2);
			int rc = selectParamTable.getTable().getRowCount();
			for (int h = 0; h < rc; h++) {
				if (selectParamTable.getTable().isRowSelected(h)) {
					selectParamTable.getTable().setSelectionBackground(Color.white);

				}
			}
		}
		mainPanel.addMouseListener(this);
		return mainPanel;
	}

	private JEditorPane createCheckButtonPanel() {
		SlickerFactory sf = SlickerFactory.instance();
		JEditorPane buttonPanel = new JEditorPane();
		buttonPanel.setEditable(false);
		buttonPanel.setBackground(new Color(160, 160, 160));
		buttonPanel.addMouseListener(this);
		//JPanel buttonPanel = sf.createRoundedPanel();
		buttonPanel.setLayout(new TableLayout(new double[][] { { 30, 140 }, { 20, 20 } }));
		selectAllButton = sf.createButton("  Select All  ");
		selectAllButton.setEnabled(true);
		selectAllButton.setActionCommand("select all");
		selectAllButton.setMnemonic(KeyEvent.VK_C);
		selectAllButton.addActionListener(this);
		selectAllButton.setToolTipText("Select all the available rules");
		buttonPanel.add(selectAllButton, "1,0");
		deselectAllButton = sf.createButton("Deselect All");
		deselectAllButton.setEnabled(true);
		deselectAllButton.setActionCommand("deselect all");
		deselectAllButton.setMnemonic(KeyEvent.VK_C);
		deselectAllButton.addActionListener(this);
		deselectAllButton.setToolTipText("Deselect all the available rules");

		buttonPanel.add(deselectAllButton, "1,1");
		buttonPanel.addMouseListener(this);
		return buttonPanel;
	}

	private JPanel createCheckOptionsPanel() {

		SlickerFactory sf = SlickerFactory.instance();
		Font font = sf.createLabel("").getFont();
		JPanel checkOptionsPanel = sf.createRoundedPanel();
		checkOptionsPanel.setLayout(new TableLayout(new double[][] { { 400 }, { 30, 30, 30, 30 } }));
		TitledBorder tb = BorderFactory.createTitledBorder("Checking options");
		tb.setBorder(BorderFactory.createLineBorder(new Color(0, 100, 0)));
		checkOptionsPanel.setBorder(tb);
		JRadioButton wholeRB = sf.createRadioButton("Check all");
		wholeRB.setFont(font);
		wholeRB.setMnemonic(KeyEvent.VK_W);
		wholeRB.setActionCommand("whole");
		wholeRB.addActionListener(this);
		wholeRB.setSelected(true);

		JRadioButton failureRB = sf.createRadioButton("Check until first failure");
		failureRB.setFont(font);
		failureRB.setMnemonic(KeyEvent.VK_F);
		failureRB.setActionCommand("failure");
		failureRB.addActionListener(this);
		failureRB.setSelected(false);

		JRadioButton successRB = sf.createRadioButton("Check until first success");
		successRB.setFont(font);
		successRB.setMnemonic(KeyEvent.VK_S);
		successRB.setActionCommand("success");
		successRB.addActionListener(this);
		successRB.setSelected(false);

		ButtonGroup bg = new ButtonGroup();
		bg.add(wholeRB);
		bg.add(failureRB);
		bg.add(successRB);

		JCheckBox skipCB = sf.createCheckBox("Skip if result is known", true);
		skipCB.setFont(font);
		skipCB.setMnemonic(KeyEvent.VK_K);
		skipCB.setActionCommand("skip");
		skipCB.addActionListener(this);
		checkOptionsPanel.add(wholeRB, "0,0");
		checkOptionsPanel.add(failureRB, "0,1");
		checkOptionsPanel.add(successRB, "0,2");
		checkOptionsPanel.add(skipCB, "0,3");

		return checkOptionsPanel;
	}

	public void actionPerformed(ActionEvent ae) {
		String action = ae.getActionCommand();
		// Per actioncommand do an action:
		if (action.equals("skip")) {
			skipReady = ((JCheckBox) ae.getSource()).isSelected();
		} else if (action.equals("select all")) {
			for (int i = 0; i < rules.length; i++) {
				if (!selectedRules.contains(rules[i])) {
					selectedRules.add(rules[i]);
				}
			}
			updateRulesList(selectedIndex);
		} else if (action.equals("deselect all")) {
			selectedRules.removeAllElements();
			updateRulesList(selectedIndex);
		} else if (action.equals("whole")) {
			firstFailure = false;
			firstSuccess = false;
		} else if (action.equals("failure")) {
			firstFailure = true;
			firstSuccess = false;
		} else if (action.equals("success")) {
			firstFailure = false;
			firstSuccess = true;
		} else if (action.equals("formula")) {
			// Get the selected formula
			String selectedFormula = (String) formulaList.getSelectedItem();
			if ((selectedFormula != null) && !selectedFormula.equals("no formula")) {
				SimpleNode formula = parser.getFormula(selectedFormula);
				// Update description this
				ASTformulaDefinition form = (ASTformulaDefinition) formula;
				String description = form.getDescription();
				descriptionPane.setText(makeHTMLPage(description));
				// Update param table
				ParamData pd = createDataModel(parser.getParameters(selectedFormula));
				selectParamTable.setModel(pd);
				paramTable.put(selectedFormula, selectParamTable);

				//selectParamTable = (ParamTable)paramTable.get(selectedFormula);
			}
		}
	}

	private void setModified(boolean value) {
		modified = value;
	}

	protected ParamData createDataModel(List<FormulaParameter> items) {
		return new ParamData(items);
	}

	protected ParamTable createParamTable() {
		return new ParamTable();
	}

	protected int getParamPaneHeight() {
		return 70;
	}

	public Object[] getSettings() {
		Iterator i = parser.getVisibleFormulaNames().iterator();
		while (i.hasNext()) {
			String formulaName = (String) i.next();
			ParamData pdCurr = createDataModel(parser.getParameters(formulaName));
			selectParamTable.setModel(pdCurr);
			Map<String, String> defaults = new HashMap<String, String>();
			for (Map.Entry<String, ValueNode> item : selectParamTable.getSubstitutes(parser).getAll().entrySet()) {
				defaults.put(item.getKey(), item.getValue().asParseableDefaultValue());
			}
			parser.setDefaultValues(formulaName, defaults);
			ParamData current = createDataModel(parser.getParameters(formulaName));
			selectParamTable.setModel(current);
			paramTable.put(formulaName, selectParamTable.getSubstitutes(parser));
		}
		Object[] out = new Object[6];
		out[0] = selectedRules;
		out[1] = paramTable;
		out[2] = firstSuccess;
		out[3] = firstFailure;
		out[4] = skipReady;
		out[5] = parser;
		return out;
	}

	public void mouseClicked(MouseEvent arg0) {

		int row = selectParamTable.getTable().rowAtPoint(arg0.getPoint());
		int column = selectParamTable.getTable().columnAtPoint(arg0.getPoint());
		if ((row != -1) && (column != -1)) {
			selectParamTable.getTable().getCellEditor(row, column).addCellEditorListener(this);
		}

		selectParamTable.setSize(selectParamTable.getTable().getWidth(), selectParamTable.getTable().getHeight());
		int rc = selectParamTable.getTable().getRowCount();
		for (int h = 0; h < rc; h++) {
			if (selectParamTable.getTable().isRowSelected(h)) {
				selectParamTable.getTable().setSelectionBackground(Color.white);

			}
		}
		if ((arg0.getSource() instanceof ParamTable) || (arg0.getSource() instanceof JTable)
				|| (arg0.getSource() instanceof JEditorPane)) {
			String formulaName = selected;
			if ((formulaName == null) || (formulaName.length() == 0)) {
				JOptionPane.showMessageDialog(this, "Please select a formula first.");
				return;
			}
			Map<String, String> defaults = new HashMap<String, String>();
			for (Map.Entry<String, ValueNode> item : selectParamTable.getSubstitutes(parser).getAll().entrySet()) {
				defaults.put(item.getKey(), item.getValue().asParseableDefaultValue());

			}
			parser.setDefaultValues(formulaName, defaults);
			ParamData pd2 = createDataModel(parser.getParameters(formulaName));
			pd2.getSubstitutes(parser);
			selectParamTable.setModel(pd2);
			selectParamTable.getTable().repaint();
			SimpleNode formula = parser.getFormula(formulaName);

			// Update description this
			ASTformulaDefinition form = (ASTformulaDefinition) formula;
			String description = form.getDescription();
			description = description + " <h2> UNDERLYING LTL FORMULA: </h2> <b> "
					+ getLTLFormula(formulaName).replace("<", "&lt;").replace(">", "&gt;") + "</b>";
			descriptionPane.setText(makeHTMLPage(description));
			//paramTable.put(formulaName, selectParamTable);
			if (!selectParamTable.getTable().isEditing()) {
				if (!selectedRules.contains(selected)) {
					selectedRules.add(selected);
				}
				updateRulesList(selectedIndex);
			}
		}
	}

	public void mouseEntered(MouseEvent arg0) {
		selectParamTable.setSize(selectParamTable.getTable().getWidth(), selectParamTable.getTable().getHeight());
		int rc = selectParamTable.getTable().getRowCount();
		for (int h = 0; h < rc; h++) {
			if (selectParamTable.getTable().isRowSelected(h)) {
				selectParamTable.getTable().setSelectionBackground(Color.white);

			}
		}
		// TODO Auto-generated method stub
	}

	public void mouseExited(MouseEvent arg0) {
		selectParamTable.setSize(selectParamTable.getTable().getWidth(), selectParamTable.getTable().getHeight());
		int rc = selectParamTable.getTable().getRowCount();
		for (int h = 0; h < rc; h++) {
			if (selectParamTable.getTable().isRowSelected(h)) {
				selectParamTable.getTable().setSelectionBackground(Color.white);

			}
		}
		// TODO Auto-generated method stub
	}

	public void mousePressed(MouseEvent arg0) {
		selectParamTable.setSize(selectParamTable.getTable().getWidth(), selectParamTable.getTable().getHeight());
		int rc = selectParamTable.getTable().getRowCount();
		for (int h = 0; h < rc; h++) {
			if (selectParamTable.getTable().isRowSelected(h)) {
				selectParamTable.getTable().setSelectionBackground(Color.white);

			}
		}
		// TODO Auto-generated method stub
	}

	public void mouseReleased(MouseEvent arg0) {
		selectParamTable.setSize(selectParamTable.getTable().getWidth(), selectParamTable.getTable().getHeight());
		int rc1 = selectParamTable.getTable().getRowCount();
		for (int h = 0; h < rc1; h++) {
			if (selectParamTable.getTable().isRowSelected(h)) {
				selectParamTable.getTable().setSelectionBackground(Color.white);
			}
		}
	}

	@Override
	public void keyPressed(KeyEvent arg0) {
		int rc = selectParamTable.getTable().getRowCount();
		for (int h = 0; h < rc; h++) {
			if (selectParamTable.getTable().isRowSelected(h)) {
				selectParamTable.getTable().setSelectionBackground(Color.white);

			}
		}
		if ((arg0.getSource() instanceof ParamTable) || (arg0.getSource() instanceof JTable)) {

			String formulaName = selected;

			if ((formulaName == null) || (formulaName.length() == 0)) {
				JOptionPane.showMessageDialog(this, "Please select a formula first.");
				return;
			}

			Map<String, String> defaults = new HashMap<String, String>();
			for (Map.Entry<String, ValueNode> item : selectParamTable.getSubstitutes(parser).getAll().entrySet()) {
				defaults.put(item.getKey(), item.getValue().asParseableDefaultValue());

			}
			parser.setDefaultValues(formulaName, defaults);
			ParamData pd2 = createDataModel(parser.getParameters(formulaName));
			selectParamTable.setModel(pd2);
			SimpleNode formula = parser.getFormula(formulaName);

			// Update description this
			ASTformulaDefinition form = (ASTformulaDefinition) formula;
			String description = form.getDescription();
			description = description + "  <h2> UNDERLYING LTL FORMULA: </h2> <b> "
					+ getLTLFormula(formulaName).replace("<", "&lt;").replace(">", "&gt;") + "</b>";
			descriptionPane.setText(makeHTMLPage(description));
		}
	}

	@Override
	public void keyReleased(KeyEvent arg0) {

		int rc = selectParamTable.getTable().getRowCount();
		for (int h = 0; h < rc; h++) {
			if (selectParamTable.getTable().isRowSelected(h)) {
				selectParamTable.getTable().setSelectionBackground(Color.white);

			}
		}
		if ((arg0.getSource() instanceof ParamTable) || (arg0.getSource() instanceof JTable)
				|| (arg0.getSource() instanceof JEditorPane)) {
			String formulaName = selected;
			if ((formulaName == null) || (formulaName.length() == 0)) {
				JOptionPane.showMessageDialog(this, "Please select a formula first.");
				return;
			}
			Map<String, String> defaults = new HashMap<String, String>();
			for (Map.Entry<String, ValueNode> item : selectParamTable.getSubstitutes(parser).getAll().entrySet()) {
				defaults.put(item.getKey(), item.getValue().asParseableDefaultValue());

			}
			parser.setDefaultValues(formulaName, defaults);
			ParamData pd2 = createDataModel(parser.getParameters(formulaName));
			pd2.getSubstitutes(parser);
			selectParamTable.setModel(pd2);
			selectParamTable.getTable().repaint();
			SimpleNode formula = parser.getFormula(formulaName);

			// Update description this
			ASTformulaDefinition form = (ASTformulaDefinition) formula;
			String description = form.getDescription();
			description = description + " <h2> UNDERLYING LTL FORMULA: </h2> <b> "
					+ getLTLFormula(formulaName).replace("<", "&lt;").replace(">", "&gt;") + "</b>";
			descriptionPane.setText(makeHTMLPage(description));
			//paramTable.put(formulaName, selectParamTable);
		}

	}

	@Override
	public void keyTyped(KeyEvent arg0) {
		int rc = selectParamTable.getTable().getRowCount();
		for (int h = 0; h < rc; h++) {
			if (selectParamTable.getTable().isRowSelected(h)) {
				selectParamTable.getTable().setSelectionBackground(Color.white);

			}
		}
		if ((arg0.getSource() instanceof ParamTable) || (arg0.getSource() instanceof JTable)
				|| (arg0.getSource() instanceof JEditorPane)) {
			String formulaName = selected;
			if ((formulaName == null) || (formulaName.length() == 0)) {
				JOptionPane.showMessageDialog(this, "Please select a formula first.");
				return;
			}
			Map<String, String> defaults = new HashMap<String, String>();
			for (Map.Entry<String, ValueNode> item : selectParamTable.getSubstitutes(parser).getAll().entrySet()) {
				defaults.put(item.getKey(), item.getValue().asParseableDefaultValue());

			}
			parser.setDefaultValues(formulaName, defaults);
			ParamData pd2 = createDataModel(parser.getParameters(formulaName));
			pd2.getSubstitutes(parser);
			selectParamTable.setModel(pd2);
			selectParamTable.getTable().repaint();
			SimpleNode formula = parser.getFormula(formulaName);

			// Update description this
			ASTformulaDefinition form = (ASTformulaDefinition) formula;
			String description = form.getDescription();
			description = description + " <h2> UNDERLYING LTL FORMULA: </h2> <b> "
					+ getLTLFormula(formulaName).replace("<", "&lt;").replace(">", "&gt;") + "</b>";
			descriptionPane.setText(makeHTMLPage(description));
			//paramTable.put(formulaName, selectParamTable);
		}
	}

	private String getLTLFormula(String formulaName) {

		TreeBuilder tb = new TreeBuilder(parser, formulaName, sets /*
																	 * , log.
																	 * getLogSummary
																	 * ().
																	 * getOntologies
																	 * () null
																	 */);
		RootNode root = new RootNode();
		ParamData pdCurr = createDataModel(parser.getParameters(formulaName));
		ParamTable selectParamTable = createParamTable();
		selectParamTable.setModel(pdCurr);
		ParamData current = createDataModel(parser.getParameters(formulaName));
		selectParamTable.setModel(current);
		Substitutes sss = selectParamTable.getSubstitutes(parser);
		sss.setBinder(root);
		boolean fullfill = false;
		FormulaNode formula1 = (FormulaNode) tb.build(parser.getFormula(formulaName), sss, root);
		root.setFormula(formula1);

		return formula1.toString();

	}

	private void updateRulesList(int currselected) {
		Vector temp = new Vector();
		selectParamTable.setSize(selectParamTable.getTable().getWidth(), selectParamTable.getTable().getHeight());
		for (int i = 0; i < rules.length; i++) {
			Cell cell = new Cell();
			if (i == currselected) {
				cell.setCurrentSelected(true);
			} else {
				cell.setCurrentSelected(false);
			}
			if (selectedRules.contains(rules[i])) {
				cell.setSelected(true);
			} else {
				cell.setSelected(false);
			}
			cell.setLabel(rules[i]);
			temp.add(cell);
		}
		//if(selectedRules.contains(rules[currselected])){
		//selectParamTable.getTable().setEnabled(true);
		//}else{
		//selectParamTable.getTable().setEnabled(false);			
		//}
		rulesList.setModel(new CellModel(temp));
	}

	@Override
	public void editingCanceled(ChangeEvent arg0) {
		// TODO Auto-generated method stub
		String io = "gigi";
	}

	@Override
	public void editingStopped(ChangeEvent arg0) {
		// TODO Auto-generated method stub
		String formulaName = selected;
		if ((formulaName == null) || (formulaName.length() == 0)) {
			JOptionPane.showMessageDialog(this, "Please select a formula first.");
			return;
		}
		Map<String, String> defaults = new HashMap<String, String>();
		for (Map.Entry<String, ValueNode> item : selectParamTable.getSubstitutes(parser).getAll().entrySet()) {
			defaults.put(item.getKey(), item.getValue().asParseableDefaultValue());

		}
		parser.setDefaultValues(formulaName, defaults);
		ParamData pd2 = createDataModel(parser.getParameters(formulaName));
		pd2.getSubstitutes(parser);
		selectParamTable.setModel(pd2);
		selectParamTable.getTable().repaint();
		SimpleNode formula = parser.getFormula(formulaName);

		// Update description this
		ASTformulaDefinition form = (ASTformulaDefinition) formula;
		String description = form.getDescription();
		description = description + " <h2> UNDERLYING LTL FORMULA: </h2> <b> "
				+ getLTLFormula(formulaName).replace("<", "&lt;").replace(">", "&gt;") + "</b>";
		descriptionPane.setText(makeHTMLPage(description));

	}
}
