package org.processmining.plugins.ltlchecker;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.text.DateFormat;
import java.text.SimpleDateFormat;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.ListCellRenderer;
import javax.swing.SwingConstants;

import org.deckfour.xes.extension.std.XExtendedEvent;
import org.deckfour.xes.model.XEvent;

import com.fluxicon.slickerbox.components.GradientPanel;

/**
 * @author Christian W. Guenther (christian@deckfour.org)
 * 
 */
public class EventCellRendererChecked extends GradientPanel implements ListCellRenderer {

	/**
	 * 
	 */
	private static final long serialVersionUID = 2972282419460405894L;
	protected int height = 60;
	protected DateFormat dateFormat = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss.SSS");

	protected JLabel nameLabel;
	protected JLabel numberLabel;
	protected JLabel originatorLabel;
	protected JLabel typeLabel;
	protected JLabel timestampLabel;
	protected int[] red;

	/**
	 * @param colorTop
	 * @param colorBottom
	 */
	public EventCellRendererChecked(int[] red) {
		super(ProMColorUI.COLOR_LIST_BG, ProMColorUI.COLOR_LIST_BG_LOWER);
		this.red = red;
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		setMinimumSize(new Dimension(180, height));
		setMaximumSize(new Dimension(500, height));
		setPreferredSize(new Dimension(250, height));
		nameLabel = new JLabel("name");
		nameLabel.setOpaque(false);
		nameLabel.setForeground(ProMColorUI.COLOR_LIST_FG);
		nameLabel.setFont(nameLabel.getFont().deriveFont(13f));
		nameLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		nameLabel.setHorizontalAlignment(SwingConstants.CENTER);
		nameLabel.setHorizontalTextPosition(SwingConstants.CENTER);
		originatorLabel = new JLabel("originator");
		originatorLabel.setOpaque(false);
		originatorLabel.setForeground(ProMColorUI.COLOR_LIST_FG);
		originatorLabel.setFont(originatorLabel.getFont().deriveFont(9f));
		numberLabel = new JLabel("#");
		numberLabel.setOpaque(false);
		numberLabel.setForeground(ProMColorUI.COLOR_LIST_FG);
		numberLabel.setFont(numberLabel.getFont().deriveFont(9f));
		typeLabel = new JLabel("type");
		typeLabel.setOpaque(false);
		typeLabel.setForeground(ProMColorUI.COLOR_LIST_FG);
		typeLabel.setFont(typeLabel.getFont().deriveFont(11f));
		timestampLabel = new JLabel("timestamp");
		timestampLabel.setOpaque(false);
		timestampLabel.setForeground(ProMColorUI.COLOR_LIST_FG);
		timestampLabel.setFont(timestampLabel.getFont().deriveFont(12f));
		timestampLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		timestampLabel.setHorizontalAlignment(SwingConstants.CENTER);
		timestampLabel.setHorizontalTextPosition(SwingConstants.CENTER);
		JPanel middlePanel = new JPanel();
		middlePanel.setLayout(new BoxLayout(middlePanel, BoxLayout.X_AXIS));
		middlePanel.setOpaque(false);
		middlePanel.setBorder(BorderFactory.createEmptyBorder());
		middlePanel.add(Box.createHorizontalGlue());
		middlePanel.add(numberLabel);
		middlePanel.add(Box.createHorizontalStrut(10));
		middlePanel.add(typeLabel);
		middlePanel.add(Box.createHorizontalStrut(10));
		middlePanel.add(originatorLabel);
		middlePanel.add(Box.createHorizontalGlue());
		this.add(Box.createVerticalGlue());
		this.add(nameLabel);
		this.add(Box.createVerticalGlue());
		this.add(middlePanel);
		this.add(Box.createVerticalGlue());
		this.add(timestampLabel);
		this.add(Box.createVerticalGlue());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * javax.swing.ListCellRenderer#getListCellRendererComponent(javax.swing
	 * .JList, java.lang.Object, int, boolean, boolean)
	 */
	public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected,
			boolean cellHasFocus) {
		// configure yourself
		if (isSelected) {
			//	this.setColors(LogPreviewUI.COLOR_LIST_SELECTION_BG, LogPreviewUI.COLOR_LIST_SELECTION_BG_LOWER);\
			setColors(ProMColorUI.COLOR_LIST_BG, ProMColorUI.COLOR_LIST_BG_LOWER);
		} else {
			setColors(ProMColorUI.COLOR_LIST_BG, ProMColorUI.COLOR_LIST_BG_LOWER);
		}
		XExtendedEvent ate = new XExtendedEvent((XEvent) value);
		nameLabel.setText(ate.getName());
		typeLabel.setText(ate.getTransition());
		numberLabel.setText("#" + (index + 1));
		for (int j = 0; j < red.length; j++) {
			if (index == red[j]) {
				numberLabel.setForeground(Color.red);
			} else {
				numberLabel.setForeground(nameLabel.getForeground());
			}
		}
		if (ate.getResource() != null) {
			originatorLabel.setText("@" + ate.getResource());
		} else {
			originatorLabel.setText("");
		}
		if (ate.getTimestamp() != null) {
			timestampLabel.setText(dateFormat.format(ate.getTimestamp()));
		} else {
			timestampLabel.setText("- no timestamp -");
		}
		//this.setBorder(BorderFactory.createLineBorder(Color.green));
		return this;
	}

}