package org.processmining.plugins.ltlchecker.importing;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintStream;

import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileNameExtensionFilter;

import org.processmining.contexts.uitopia.annotations.UIImportPlugin;
import org.processmining.framework.abstractplugins.AbstractImportPlugin;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.plugins.ltlchecker.model.LTLModel;
import org.processmining.plugins.ltlchecker.parser.LTLParser;

@Plugin(name = "Import LTL Model", parameterLabels = { "Filename" }, returnLabels = { "LTL Model" }, returnTypes = { LTLModel.class })
@UIImportPlugin(description = "LTL Model", extensions = { "ltl" })
public class LTLModelImportPlugin extends AbstractImportPlugin {

	protected FileFilter getFileFilter() {
		return new FileNameExtensionFilter("LTL files", "ltl");
	}

	protected LTLModel importFromStream(PluginContext context, InputStream input, String filename, long fileSizeInBytes)
			throws Exception {
		context.getFutureResult(0).setLabel("LTL Model imported from " + filename);
		LTLModel result = new LTLModel();
		String fileStr = "";
		BufferedReader br = new BufferedReader(new InputStreamReader(input));
		String line;
		try {
			line = br.readLine();
			while (line != null) {
				fileStr = fileStr + line + "\n";
				line = br.readLine();
			}
			result.setFile(fileStr);
			FileInputStream fis = null;
			LTLParser newparser = null;
			File fileLTL = null;
			fileLTL = File.createTempFile("tempLTL", ".ltl");
			PrintStream out = new PrintStream(fileLTL);
			out.println(fileStr);
			fis = new FileInputStream(fileLTL.getAbsolutePath());
			newparser = new LTLParser(fis);
			newparser.setFilename(fileLTL.getAbsolutePath());
			newparser.init();
			newparser.parse();
			if (fis != null) {
				fis.close();
			}
			fileLTL.delete();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (Exception e) {
			throw new Exception("Parser Exception: " + e.getMessage());
		}
		return result;

	}
}